#!/usr/bin/python3
# vim:et:sta:sts=4:sw=4:ts=8:tw=79:

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
import os
import sys
import subprocess
import datetime
import pwd
import grp


class SystemUsers:

    def all_users_data(self):
        userdata = []
        for i in pwd.getpwall():
            login = i[0]
            uid = i[2]
            gid = i[3]
            gecos = i[4]
            homedir = i[5]
            shell = i[6]
            if ',' in gecos:
                fullname = gecos.partition(',')[0]
                officeinfo = gecos.partition(',')[2].partition(',')[0]
                officeext = gecos.partition(',')[2].partition(',')[
                    2].partition(',')[0]
                homephone = gecos.partition(',')[2].partition(
                    ',')[2].partition(',')[2].partition(',')[0]
            else:
                fullname = gecos
                officeinfo = ''
                officeext = ''
                homephone = ''
            userdata.append([login, uid, gid, fullname, officeinfo,
                             officeext, homephone, homedir, shell])
        return userdata

    def default_groups(self):
        groups = []
        groups_to_try = ('audio', 'video', 'cdrom', 'floppy',
                         'lp', 'plugdev', 'polkitd', 'pulse',
                         'scanner', 'power', 'netdev')
        for i in groups_to_try:
            try:
                groups.append(grp.getgrnam(i)[0])
            except KeyError:
                pass

        return groups

    def user_list(self, all=False):
        userlist = []
        for i in self.all_users_data():
            if all == False:
                if i[1] >= 500:
                    userlist.append(i)
            else:
                userlist.append(i)
        return userlist

    def get_uid(self, login):
        uid = pwd.getpwnam(login)[2]
        return uid

    def get_gid(self, group):
        gid = grp.getgrnam(group)[2]
        return gid

    def get_fullname(self, login):
        for i in self.all_users_data():
            if i[0] == login:
                fullname = i[3]
        return fullname

    def get_officeinfo(self, login):
        for i in self.all_users_data():
            if i[0] == login:
                officeinfo = i[4]
        return officeinfo

    def get_officeext(self, login):
        for i in self.all_users_data():
            if i[0] == login:
                officeext = i[5]
        return officeext

    def get_homephone(self, login):
        for i in self.all_users_data():
            if i[0] == login:
                homephone = i[6]
        return homephone

    def get_homedir(self, login):
        homedir = pwd.getpwnam(login)[5]
        return homedir

    def get_shell(self, login):
        shell = pwd.getpwnam(login)[6]
        return shell

    def get_maingroup(self, login):
        group = subprocess.getoutput(
            'groups ' + login).partition(' : ')[2].partition(' ')[0]
        return group

    def get_groups(self, login):
        groups = []
        allgroups = grp.getgrall()
        for i in allgroups:
            if login in i[3]:
                groups.append(i[0])
        return groups

    def get_expiry_date(self, login):
        now = datetime.datetime.now()
        expirydays = subprocess.getoutput(
            'grep -e "^' + login + ':.*$" /etc/shadow | cut -d : -f 8')
        if expirydays == '':
            state = False
            year = now.year
            month = now.month
            day = now.day
        else:
            state = True
            expirydate = subprocess.getoutput(
                'date -u --date "Jan 1, 1970 + ' + expirydays + ' days" +%Y-%m-%d')
            year = int(expirydate.partition('-')[0])
            month = int(expirydate.partition('-')[2].partition('-')[0]) - 1
            day = int(expirydate.partition('-')
                      [2].partition('-')[2].partition('-')[0])
        return state, year, month, day

    def create_user(self, login, shell, group, groups, password):
        os.system('useradd -s ' + shell + ' -g ' +
                  group + ' -m -k /etc/skel ' + login)
        res = self.set_password(login, password)
        if res["error"]:
            # I wish I didn't have to do this. But the only way for PAM and
            # chpasswd to report a "BAD PASSWORD" is to first create the
            # account and then try to change it. So, we have to delete the
            # newly created account if that happens.
            os.system('userdel -r ' + login)
            return res
        for i in groups:
            self.add_user_to_group(login, i)
        return res

    def create_group(self, groupname, gid):
        os.system('groupadd -g ' + str(gid) + ' ' + groupname)

    def del_user(self, login, remove_home=False):
        if remove_home == True:
            homedir = self.get_homedir(login)
            os.system('rm -rf ' + homedir)
        os.system('userdel ' + login)

    def del_group(self, groupname):
        os.system('groupdel ' + groupname)

    def set_password(self, login, password):
        d = { "error": False, "message": None }
        res = subprocess.getoutput('echo "' + login + ':' + password + '" | chpasswd')
        if len(res) > 0:
            d["error"] = True
            d["message"] = res
        return d

    def set_uid(self, login, uid):
        os.system('usermod -u ' + str(uid) + ' ' + login)

    def set_gid(self, group, gid):
        os.system('groupmod -g ' + str(gid) + ' ' + group)

    def set_group(self, login, group):
        os.system('usermod -g ' + group + ' ' + login)

    def add_user_to_group(self, login, group):
        os.system('gpasswd -a ' + login + ' ' + group + ' > /dev/null')

    def remove_user_from_group(self, login, group):
        os.system('gpasswd -d ' + login + ' ' + group + ' > /dev/null')

    def set_fullname(self, login, fullname):
        fullname = '"' + fullname + '"'
        os.system('chfn -f ' + fullname + ' ' + login)

    def set_officeinfo(self, login, officeinfo):
        officeinfo = '"' + officeinfo + '"'
        os.system('chfn -o ' + officeinfo + ' ' + login)

    def set_officeext(self, login, officeext):
        officeext = '"' + officeext + '"'
        os.system('chfn -p ' + officeext + ' ' + login)

    def set_homephone(self, login, homephone):
        homephone = '"' + homephone + '"'
        os.system('chfn -h ' + homephone + ' ' + login)

    def set_homedir(self, login, homedir):
        os.system('usermod -d ' + homedir + ' ' + login)

    def set_shell(self, login, shell):
        os.system('usermod -s ' + shell + ' ' + login)

    def set_expiry_date(self, login, year, month, day):
        expirydate = str(year) + '-' + str(month) + '-' + str(day)
        os.system('usermod -e ' + expirydate + ' ' + login)

    def set_no_expiry_date(self, login):
        os.system('usermod -e "" ' + login)

    def get_all_groups(self):
        groups = []
        for i in grp.getgrall():
            groupname = i[0]
            gid = i[2]
            members = i[3]
            groups.append([groupname, gid, members])
        return groups

    def user_in_group(self, login, group):
        if group in self.get_groups(login):
            member = True
        else:
            member = False
        return member

    def check_password(self, password1, password2):
        if password1 == password2:
            if len(password1) < 6:
                setpassword = False
                msg = _(
                    "Password is too short. It has to be at least 5 characters long.")
            elif '\\' in password1:
                setpassword = False
                msg = _("Password cannot include the backslash character.")
            else:
                setpassword = True
                msg = ''
        else:
            setpassword = False
            msg = _("Passwords don't match")
        return setpassword, msg

    def check_username(self, username):
        u = SystemUsers()
        setusername = True
        msg = ''
        if username == '':
            setusername = False
            msg = _("Username cannot be empty.")
        elif len(username) > 31:
            setusername = False
            msg = _("Username cannot exceed 31 characters")
        elif ' ' in username:
            setusername = False
            msg = _("Username cannot include the space character.")
        elif username.startswith(('0', '1', '2', '3', '4', '5', '6', '7', '8', '9')):
            setusername = False
            msg = _("Username cannot start with a number.")
        elif username.startswith('_'):
            setusername = False
            msg = _("Username cannot start with an underscore.")
        elif username.startswith('-'):
            setusername = False
            msg = _("Username cannot start with a dash.")
        if setusername == True:
            for i in username:
                if not i in 'abcdefghijklmnopqrstuvwxyz-_0123456789':
                    setusername = False
                    msg = _(
                        "Username can consist solely of latin lower case letters, numbers, dashes and underscores.")
                    break
        if setusername == True:
            for i in u.user_list():
                if username == i[0]:
                    setusername = False
                    msg = _("Username \"{0}\" already exists.").format(
                        username)
                    break
        return setusername, msg

    def check_groupname(self, groupname):
        g = SystemUsers()
        setgroupname = True
        msg = ''
        if groupname == '':
            setgroupname = False
            msg = _("Group name cannot be empty.")
        elif len(groupname) > 31:
            setgroupname = False
            msg = _("Group name cannot exceed 31 characters")
        elif ' ' in groupname:
            setgroupname = False
            msg = _("Group name cannot include the space character.")
        elif groupname.startswith(('0', '1', '2', '3', '4', '5', '6', '7', '8', '9')):
            setgroupname = False
            msg = _("Group name cannot start with a number.")
        elif groupname.startswith('_'):
            setgroupname = False
            msg = _("Group name cannot start with an underscore.")
        elif groupname.startswith('-'):
            setgroupname = False
            msg = _("Group name cannot start with a dash.")
        if setgroupname == True:
            for i in groupname:
                if not i in 'abcdefghijklmnopqrstuvwxyz-_0123456789':
                    setgroupname = False
                    msg = _(
                        "Group name can consist solely of latin lower case letters, numbers, dashes and underscores.")
                    break
        if setgroupname == True:
            for i in g.get_all_groups():
                if groupname == i[0]:
                    setgroupname = False
                    msg = _("Group name \"{0}\" already exists.").format(
                        groupname)
                    break
        return setgroupname, msg

    def check_shell(self, shell):
        if os.path.exists(shell):
            setshell = True
            msg = ''
        else:
            setshell = False
            msg = _("Shell does not exist.")
        return setshell, msg

    def check_uid(self, uid):
        u = SystemUsers()
        setuid = True
        msg = ''
        for i in u.all_users_data():
            if uid == i[1]:
                setuid = False
                msg = _("UID {0} is assigned to another user.").format(uid)
                break
        return setuid, msg

    def check_gid(self, gid):
        setgid = True
        msg = ''
        for i in grp.getgrall():
            if i[2] == gid:
                setgid = False
                msg = _("GID {0} is assigned to another group.").format(gid)
                break
        return setgid, msg

# Internationalization
import locale
import gettext
locale.setlocale(locale.LC_ALL, '')
locale.bindtextdomain("gtkusersetup", "/usr/share/locale")
gettext.bindtextdomain("gtkusersetup", "/usr/share/locale")
gettext.textdomain("gtkusersetup")
_ = gettext.gettext


class GTKUserSetup:

    def on_button_about_clicked(self, widget, data=None):
        self.aboutdialog.show()

    def on_aboutdialog_response(self, widget, data=None):
        self.aboutdialog.hide()

    def on_aboutdialog_delete_event(self, widget, event):
        self.aboutdialog.hide()
        return True

    def gtk_main_quit(self, widget, data=None):
        Gtk.main_quit()

    def on_button_close_clicked(self, widget, data=None):
        Gtk.main_quit()

    def on_button_properties_clicked(self, widget, data=None):
        try:
            u = SystemUsers()
            self.task_add_user = False
            user = self.userlist.get_selection()
            self.userliststore, iter = user.get_selected()
            username = self.userliststore.get_value(iter, 1)
            fullname = u.get_fullname(username)
            officeinfo = u.get_officeinfo(username)
            officeext = u.get_officeext(username)
            homephone = u.get_homephone(username)
            maingroup = u.get_maingroup(username)
            homedir = u.get_homedir(username)
            shell = u.get_shell(username)
            uid = u.get_uid(username)
            expirydate = u.get_expiry_date(username)

            #
            # Set existing values
            #
            self.user_properties_window.set_title(
                _("User account properties for \"{0}\"").format(username))
            self.notebook_userproperties.set_current_page(0)

            # Account tab
            self.entry_login.set_text(username)
            self.entry_login.set_sensitive(False)
            self.entry_fullname.set_text(fullname)
            self.label_passwordheader.set_label(_("<b>Set new password</b>"))
            self.entry_password1.set_text('')
            self.entry_password2.set_text('')

            # Contact tab
            self.entry_officeinfo.set_text(officeinfo)
            self.entry_officeext.set_text(officeext)
            self.entry_homephone.set_text(homephone)

            # Groups tab
            allgroups = u.get_all_groups()
            self.groupliststore.clear()
            count = 0
            for i in allgroups:
                self.groupliststore.append([i[0]])
                if i[0] == maingroup:
                    self.combobox_maingroup.set_active(count)
                count += 1
            self.user_groupliststore.clear()
            for i in allgroups:
                group = i[0]
                gid = i[1]
                groupmembers = i[2]
                if u.user_in_group(username, group):
                    ismember = True
                else:
                    ismember = False
                self.user_groupliststore.append([ismember, group, gid, False])

            # Advanced tab
            self.label_homedir.show()
            self.entry_homedir.show()
            self.entry_homedir.set_text(homedir)
            self.entry_shell.set_text(shell)
            self.label_uid.show()
            self.spinbutton_uid.show()
            self.spinbutton_uid.set_value(uid)
            if expirydate[0] == False:
                self.switch_user_expiration.set_active(False)
                self.calendar_user_expiration.set_sensitive(False)
            else:
                self.switch_user_expiration.set_active(True)
                self.calendar_user_expiration.set_sensitive(True)
            self.calendar_user_expiration.select_month(
                expirydate[2] - 1, expirydate[1])
            self.calendar_user_expiration.select_day(expirydate[3])

            self.user_properties_window.show()
        except TypeError:
            pass

    def on_button_adduser_clicked(self, widget):
        u = SystemUsers()
        now = datetime.datetime.now()
        self.task_add_user = True
        #
        # Set existing values
        #
        self.user_properties_window.set_title(_("Add new user"))
        self.notebook_userproperties.set_current_page(0)

        # Account tab
        self.entry_login.set_sensitive(True)
        self.entry_login.set_text('')
        self.entry_fullname.set_text('')
        self.label_passwordheader.set_label(_("<b>Set password</b>"))
        self.entry_password1.set_text('')
        self.entry_password2.set_text('')

        # Contact tab
        self.entry_officeinfo.set_text('')
        self.entry_officeext.set_text('')
        self.entry_homephone.set_text('')

        # Groups tab
        allgroups = u.get_all_groups()
        defaultgroups = u.default_groups()
        self.groupliststore.clear()
        count = 0
        for i in allgroups:
            self.groupliststore.append([i[0]])
            if i[0] == 'users':
                self.combobox_maingroup.set_active(count)
            count += 1
        self.user_groupliststore.clear()
        for i in allgroups:
            group = i[0]
            gid = i[1]
            groupmembers = i[2]
            if group in defaultgroups:
                ismember = True
            else:
                ismember = False
            self.user_groupliststore.append([ismember, group, gid, False])

        # Advanced tab
        self.label_homedir.hide()
        self.entry_homedir.hide()
        self.entry_shell.set_text('/bin/bash')
        self.label_uid.hide()
        self.spinbutton_uid.hide()
        self.switch_user_expiration.set_active(False)
        self.calendar_user_expiration.set_sensitive(False)
        self.calendar_user_expiration.select_month(now.month - 1, now.year)
        self.calendar_user_expiration.select_day(now.day)

        self.user_properties_window.show()

    def on_button_deleteuser_clicked(self, widget):
        try:
            u = SystemUsers()
            user = self.userlist.get_selection()
            self.userliststore, iter = user.get_selected()
            username = self.userliststore.get_value(iter, 1)
            self.switch_delete_user_homedir.set_active(False)
            self.delete_user_window.set_title(
                _("Delete user {0}?").format(username))
            self.label_delete_user.set_text(
                _("You are about to remove user account \"{0}\" from your system. Do you really want to continue?").format(username))
            self.label_delete_user_homedir.set_label(
                _("Completely remove user's home directory ({0}) with all its contents").format(u.get_homedir(username)))
            self.delete_user_window.show()
        except TypeError:
            pass

    def on_button_managegroups_clicked(self, widget):
        self.refresh_group_list()
        self.group_settings_window.show()

    def on_switch_showall_toggled(self, widget, data=None):
        self.userliststore.clear()
        status = self.switch_showall.get_active()
        u = SystemUsers()
        for i in u.user_list(status):
            self.userliststore.append([i[3], i[0], i[7], i[1]])

    def on_button_user_properties_ok_clicked(self, widget, data=None):
        u = SystemUsers()
        if self.task_add_user:
            #
            # Adding new user
            #
            username = self.entry_login.get_text()
            fullname = self.entry_fullname.get_text()
            password1 = self.entry_password1.get_text()
            password2 = self.entry_password2.get_text()
            officeinfo = self.entry_officeinfo.get_text()
            officeext = self.entry_officeext.get_text()
            homephone = self.entry_homephone.get_text()
            maingroup_iter = self.combobox_maingroup.get_active_iter()
            maingroup_model = self.combobox_maingroup.get_model()
            maingroup = maingroup_model[maingroup_iter][0]
            shell = self.entry_shell.get_text()
            expiry = self.switch_user_expiration.get_active()

            checkusername = u.check_username(username)
            checkpassword = u.check_password(password1, password2)
            checkshell = u.check_shell(shell)
            if checkusername[0] == False:
                self.label_properties_error_msg.set_label(checkusername[1])
                self.properties_error_window.show()
            elif checkpassword[0] == False:
                self.label_properties_error_msg.set_label(checkpassword[1])
                self.properties_error_window.show()
            elif checkshell[0] == False:
                self.label_properties_error_msg.set_label(checkshell[1])
                self.properties_error_window.show()
            else:
                groups = []
                for i in self.user_groupliststore:
                    if i[0] == True:
                        groups.append(i[1])
                res = u.create_user(username, shell, maingroup, groups, password1)
                if res["error"]:
                    self.label_properties_error_msg.set_label(res["message"])
                    self.properties_error_window.show()
                    return
                u.set_fullname(username, fullname)
                u.set_officeinfo(username, officeinfo)
                u.set_officeext(username, officeext)
                u.set_homephone(username, homephone)
                if expiry:
                    year = int("%04d" %
                               (self.calendar_user_expiration.get_date()[0]))
                    month = int(
                        "%02d" % (self.calendar_user_expiration.get_date()[1])) + 1
                    day = int("%02d" %
                              (self.calendar_user_expiration.get_date()[2]))
                    u.set_expiry_date(username, year, month, day)

                self.user_properties_window.hide()
                self.on_switch_showall_toggled(self)
        else:
            #
            # Editing existing user info
            #
            username = self.entry_login.get_text()
            fullname = u.get_fullname(username)
            officeinfo = u.get_officeinfo(username)
            officeext = u.get_officeext(username)
            homephone = u.get_homephone(username)
            maingroup = u.get_maingroup(username)
            homedir = u.get_homedir(username)
            shell = u.get_shell(username)
            uid = u.get_uid(username)
            expiry_date = u.get_expiry_date(username)

            new_fullname = self.entry_fullname.get_text()
            new_password1 = self.entry_password1.get_text()
            new_password2 = self.entry_password2.get_text()
            new_officeinfo = self.entry_officeinfo.get_text()
            new_officeext = self.entry_officeext.get_text()
            new_homephone = self.entry_homephone.get_text()
            new_maingroup_iter = self.combobox_maingroup.get_active_iter()
            new_maingroup_model = self.combobox_maingroup.get_model()
            new_maingroup = new_maingroup_model[new_maingroup_iter][0]
            new_shell = self.entry_shell.get_text()
            new_uid = self.spinbutton_uid.get_value_as_int()
            new_expiry_date = self.switch_user_expiration.get_active()

            bool_raise_error = False
            bool_set_fullname = False
            bool_set_password = False
            bool_set_officeinfo = False
            bool_set_officeext = False
            bool_set_homephone = False
            bool_set_maingroup = False
            bool_set_shell = False
            bool_set_uid = False
            bool_set_expirydate = False

            if not new_fullname == fullname:
                u.set_fullname(username, new_fullname)

            if not new_password1 == "":
                checkpass = u.check_password(new_password1, new_password2)
                if checkpass[0] == True:
                    bool_set_password = True
                else:
                    bool_raise_error = True
                    self.label_properties_error_msg.set_label(checkpass[1])
                    self.properties_error_window.show()

            if not new_officeinfo == officeinfo:
                bool_set_officeinfo = True

            if not new_officeext == officeext:
                bool_set_officeext = True

            if not new_homephone == homephone:
                bool_set_homephone = True

            if not new_maingroup == maingroup:
                bool_set_maingroup = True

            if not new_shell == shell:
                checkshell = u.check_shell(new_shell)
                if checkshell[0] == True:
                    bool_set_shell = True
                else:
                    bool_raise_error = True
                    self.label_properties_error_msg.set_label(checkshell[1])
                    self.properties_error_window.show()

            if not new_uid == uid:
                checkuid = u.check_uid(new_uid)
                if checkuid[0] == True:
                    bool_set_uid = True
                else:
                    bool_set_uid = False
                    bool_raise_error = True
                    self.label_properties_error_msg.set_label(checkuid[1])
                    self.properties_error_window.show()

            if bool_raise_error == False:
                if bool_set_fullname == True:
                    u.set_fullname(username, new_fullname)
                if bool_set_password == True:
                    result = u.set_password(username, new_password1)
                    if result["error"]:
                        # display message
                        self.label_properties_error_msg.set_label(result["message"])
                        self.properties_error_window.show()
                        return
                if bool_set_officeinfo == True:
                    u.set_officeinfo(username, new_officeinfo)
                if bool_set_officeext == True:
                    u.set_officeext(username, new_officeext)
                if bool_set_homephone == True:
                    u.set_homephone(username, new_homephone)
                if bool_set_maingroup == True:
                    u.set_group(username, new_maingroup)
                if bool_set_shell == True:
                    u.set_shell(username, new_shell)
                if bool_set_uid == True:
                    u.set_uid(username, new_uid)
                if new_expiry_date == True:
                    year = int("%04d" %
                               (self.calendar_user_expiration.get_date()[0]))
                    month = int(
                        "%02d" % (self.calendar_user_expiration.get_date()[1])) + 1
                    day = int("%02d" %
                              (self.calendar_user_expiration.get_date()[2]))
                    u.set_expiry_date(username, year, month, day)
                else:
                    if expiry_date[0] == True:
                        u.set_no_expiry_date(username)
                for i in self.user_groupliststore:
                    if i[3] == True:
                        if i[0] == True:
                            u.add_user_to_group(username, i[1])
                        else:
                            u.remove_user_from_group(username, i[1])
                self.user_properties_window.hide()
                self.on_switch_showall_toggled(self)

    def on_button_user_properties_cancel_clicked(self, widget, data=None):
        self.user_properties_window.hide()

    def on_group_checkbox_toggled(self, widget, data=None):
        selectedgroup = self.user_grouplist.get_selection()
        self.user_groupliststore, iter = selectedgroup.get_selected()
        selectedgroupname = self.user_groupliststore.get_value(iter, 1)
        for i in self.user_groupliststore:
            if i[1] == selectedgroupname:
                i[3] = True
                if i[0] == 0:
                    i[0] = 1
                else:
                    i[0] = 0

    def on_switch_visible_password_toggled(self, widget, data=None):
        state = self.entry_password1.get_visibility()
        if state == False:
            self.entry_password1.set_visibility(True)
            self.entry_password2.set_visibility(True)
        else:
            self.entry_password1.set_visibility(False)
            self.entry_password2.set_visibility(False)

    def on_switch_user_expiration_toggled(self, widget, data=None):
        state = self.switch_user_expiration.get_active()
        self.calendar_user_expiration.set_sensitive(state)

    def on_user_properties_window_delete_event(self, widget, event):
        self.user_properties_window.hide()
        return True

    def on_button_error_window_ok_clicked(self, widget, data=None):
        self.properties_error_window.hide()

    def on_properties_error_window_delete_event(self, widget, event):
        self.properties_error_window.hide()
        return True

    def on_button_deleteuser_ok_clicked(self, widget):
        u = SystemUsers()
        user = self.userlist.get_selection()
        self.userliststore, iter = user.get_selected()
        username = self.userliststore.get_value(iter, 1)
        delete_homedir = self.switch_delete_user_homedir.get_active()
        u.del_user(username, delete_homedir)
        self.delete_user_window.hide()
        self.on_switch_showall_toggled(self)

    def on_button_deleteuser_cancel_clicked(self, widget):
        self.delete_user_window.hide()

    def on_delete_user_window_delete_event(self, widget, event):
        self.delete_user_window.hide()
        return True

    def on_button_group_settings_close_clicked(self, widget):
        self.group_settings_window.hide()

    def on_group_settings_window_delete_event(self, widget, event):
        self.group_settings_window.hide()
        return True

    def on_button_group_properties_clicked(self, widget):
        g = SystemUsers()
        self.group_task_add_group = False
        group = self.grouplist.get_selection()
        self.groupliststore, iter = group.get_selected()
        groupname = self.groupliststore.get_value(iter, 0)
        self.group_properties_window.set_title(
            _("Group \"{0}\" properties").format(groupname))
        self.entry_groupname.set_sensitive(False)
        self.entry_groupname.set_text(groupname)
        self.spinbutton_gid.set_value(g.get_gid(groupname))
        self.groupmembersliststore.clear()
        for i in g.user_list(True):
            user = i[0]
            if g.user_in_group(user, groupname):
                member = True
            else:
                member = False
            self.groupmembersliststore.append([member, user, False])
        self.group_properties_window.show()

    def on_button_group_add_clicked(self, widget):
        g = SystemUsers()
        self.group_task_add_group = True
        self.entry_groupname.set_sensitive(True)
        self.entry_groupname.set_text('')
        self.group_properties_window.set_title(_("Add new group"))
        gidlist = []
        for i in grp.getgrall():
            gidlist.append(int(i[2]))
        maxgid = max(gidlist)
        if maxgid >= 65535:
            maxgid = 1000
        self.spinbutton_gid.set_value(maxgid + 1)
        self.groupmembersliststore.clear()
        for i in g.user_list(True):
            user = i[0]
            self.groupmembersliststore.append([False, user, False])
        self.group_properties_window.show()

    def on_button_group_delete_clicked(self, widget):
        g = SystemUsers()
        group = self.grouplist.get_selection()
        self.groupliststore, iter = group.get_selected()
        groupname = self.groupliststore.get_value(iter, 0)
        bool_del_group = True
        for i in g.user_list(True):
            if g.get_maingroup(i[0]) == groupname:
                bool_del_group = False
                break
        if bool_del_group == True:
            self.label_delete_group.set_text(
                _("You are about to delete the \"{0}\" group from your system. Are you sure you want to continue?").format(groupname))
            self.delete_group_window.show()
        else:
            self.label_cannot_delete_group.set_text(
                _("Group \"{0}\" cannot be deleted as it is the main group for user \"{1}\". If you want to delete group \"{0}\", you have to set the main group for user \"{1}\" to another group first.").format(groupname, i[0]))
            self.cannot_delete_group_window.show()

    def refresh_group_list(self):
        allgroups = grp.getgrall()
        self.groupliststore.clear()
        count = 0
        for i in allgroups:
            self.groupliststore.append([i[0]])

    def on_button_group_properties_ok_clicked(self, widget):
        g = SystemUsers()
        bool_raise_error = False
        if self.group_task_add_group == True:
            # Adding a group
            group = self.entry_groupname.get_text()
            gid = self.spinbutton_gid.get_value_as_int()
            checkgroupname = g.check_groupname(group)
            if checkgroupname[0]:
                checkgid = g.check_gid(gid)
                if checkgid[0]:
                    g.create_group(group, gid)
                else:
                    bool_raise_error = True
                    self.label_group_error_msg.set_label(checkgid[1])
                    self.group_properties_error_window.show()
            else:
                bool_raise_error = True
                self.label_group_error_msg.set_label(
                    g.check_groupname(group)[1])
                self.group_properties_error_window.show()
            if bool_raise_error == False:
                for i in self.groupmembersliststore:
                    if i[0] == True:
                        g.add_user_to_group(i[1], group)
                self.refresh_group_list()
                self.group_properties_window.hide()
        else:
            # Editing an existing group
            group = self.entry_groupname.get_text()
            gid = g.get_gid(group)
            new_gid = self.spinbutton_gid.get_value_as_int()
            if not new_gid == gid:
                checkgid = g.check_gid(new_gid)
                if checkgid[0]:
                    g.set_gid(group, new_gid)
                    self.group_properties_window.hide()
                else:
                    bool_raise_error = True
                    self.label_group_error_msg.set_label(checkgid[1])
                    self.group_properties_error_window.show()
            if bool_raise_error == False:
                for i in self.groupmembersliststore:
                    if i[2] == True:
                        if i[0] == False:
                            g.remove_user_from_group(i[1], group)
                        else:
                            g.add_user_to_group(i[1], group)
                self.refresh_group_list()
                self.group_properties_window.hide()

    def on_button_group_properties_cancel_clicked(self, widget):
        self.group_properties_window.hide()

    def on_group_properties_window_delete_event(self, widget, event):
        self.group_properties_window.hide()
        return True

    def on_groupmembers_checkbox_toggled(self, widget, data=None):
        selecteduser = self.groupmembers.get_selection()
        self.groupmembersliststore, iter = selecteduser.get_selected()
        selectedusername = self.groupmembersliststore.get_value(iter, 1)
        for i in self.groupmembersliststore:
            if i[1] == selectedusername:
                i[2] = True
                if i[0] == False:
                    i[0] = True
                else:
                    i[0] = False

    def on_button_group_error_ok_clicked(self, widget):
        self.group_properties_error_window.hide()

    def on_group_properties_error_window_delete_event(self, widget, event):
        self.group_properties_error_window.hide()
        return True

    def on_button_delete_group_ok_clicked(self, widget):
        g = SystemUsers()
        group = self.grouplist.get_selection()
        self.groupliststore, iter = group.get_selected()
        groupname = self.groupliststore.get_value(iter, 0)
        g.del_group(groupname)
        self.refresh_group_list()
        self.delete_group_window.hide()

    def on_button_delete_group_cancel_clicked(self, widget):
        self.delete_group_window.hide()

    def on_delete_group_window_delete_event(self, widget, event):
        self.delete_group_window.hide()
        return True

    def on_button_cannot_delete_group_ok_clicked(self, widget):
        self.cannot_delete_group_window.hide()

    def on_cannot_delete_group_window_delete_event(self, widget, event):
        self.cannot_delete_group_window.hide()
        return True

    def __init__(self):
        builder = Gtk.Builder()
        builder.set_translation_domain("gtkusersetup")
        if os.path.exists('gtkusersetup.ui'):
            builder.add_from_file('gtkusersetup.ui')
        elif os.path.exists('/usr/share/salixtools/gtkusersetup/gtkusersetup.ui'):
            builder.add_from_file(
                '/usr/share/salixtools/gtkusersetup/gtkusersetup.ui')

        #
        # Main window objects
        #
        self.window = builder.get_object('gtkusersetup')
        self.namecolumn = builder.get_object('namecolumn')
        self.logincolumn = builder.get_object('logincolumn')
        self.homedircolumn = builder.get_object('homedircolumn')
        self.userlist = builder.get_object('userlist')
        self.userliststore = builder.get_object('userliststore')
        self.switch_showall = builder.get_object('switch_showall')
        self.image_manageuser = builder.get_object('image_manageuser')
        self.logincolumn.set_title(_('Username'))
        self.namecolumn.set_title(_('Real name'))
        self.homedircolumn.set_title(_('Home directory'))

        self.on_switch_showall_toggled(self)
        self.userlist.set_cursor(1)

        #
        # About dialog
        #
        self.aboutdialog = builder.get_object('aboutdialog')

        #
        # User properties/Add user window objects
        #
        self.notebook_userproperties = builder.get_object(
            'notebook_userproperties')
        self.user_properties_window = builder.get_object(
            'user_properties_window')

        # Account tab
        self.entry_login = builder.get_object('entry_login')
        self.entry_fullname = builder.get_object('entry_fullname')
        self.label_passwordheader = builder.get_object('label_passwordheader')
        self.entry_password1 = builder.get_object('entry_password1')
        self.entry_password2 = builder.get_object('entry_password2')
        self.switch_visible_password = builder.get_object(
            'switch_visible_password')

        # Contact tab
        self.entry_officeinfo = builder.get_object('entry_officeinfo')
        self.entry_officeext = builder.get_object('entry_officeext')
        self.entry_homephone = builder.get_object('entry_homephone')

        # Groups tab
        self.combobox_maingroup = builder.get_object('combobox_maingroup')
        self.groupliststore = builder.get_object('groupliststore')
        self.user_grouplist = builder.get_object('user_grouplist')
        self.user_groupliststore = builder.get_object('user_groupliststore')
        self.column_user_membership_checked = builder.get_object(
            'column_user_membership_checked')
        self.column_user_membership_group = builder.get_object(
            'column_user_membership_group')
        self.column_user_membership_gid = builder.get_object(
            'column_user_membership_gid')
        self.column_user_membership_checked.set_title(_('Member'))
        self.column_user_membership_group.set_title(_('Group'))
        self.column_user_membership_gid.set_title('GID')

        # Advanced tab
        self.label_homedir = builder.get_object('label_homedir')
        self.entry_homedir = builder.get_object('entry_homedir')
        self.entry_shell = builder.get_object('entry_shell')
        self.label_uid = builder.get_object('label_uid')
        self.spinbutton_uid = builder.get_object('spinbutton_uid')
        self.switch_user_expiration = builder.get_object(
            'switch_user_expiration')
        self.calendar_user_expiration = builder.get_object(
            'calendar_user_expiration')

        #
        # Error message for user properties window
        #
        self.properties_error_window = builder.get_object(
            'properties_error_window')
        self.button_error_window_ok = builder.get_object(
            'button_error_window_ok')
        self.label_properties_error_msg = builder.get_object(
            'label_properties_error_msg')

        #
        # Delete user message confirmation
        #
        self.delete_user_window = builder.get_object('delete_user_window')
        self.label_delete_user = builder.get_object('label_delete_user')
        self.switch_delete_user_homedir = builder.get_object(
            'switch_delete_user_homedir')
        self.label_delete_user_homedir = builder.get_object(
            'label_delete_user_homedir')

        #
        # Main group settings window
        #
        self.group_settings_window = builder.get_object(
            'group_settings_window')

        #
        # Group properties window
        #
        self.group_properties_window = builder.get_object(
            'group_properties_window')
        self.grouplist = builder.get_object('grouplist')
        self.label_groupname = builder.get_object('label_groupname')
        self.entry_groupname = builder.get_object('entry_groupname')
        self.spinbutton_gid = builder.get_object('spinbutton_gid')
        self.groupmembers = builder.get_object('groupmembers')
        self.groupmembersliststore = builder.get_object(
            'groupmembersliststore')

        #
        # Group properties error window
        #
        self.group_properties_error_window = builder.get_object(
            'group_properties_error_window')
        self.label_group_error_msg = builder.get_object(
            'label_group_error_msg')

        #
        # Delete group message confirmation
        #
        self.delete_group_window = builder.get_object('delete_group_window')
        self.label_delete_group = builder.get_object('label_delete_group')

        #
        # Cannot delete group window
        #
        self.cannot_delete_group_window = builder.get_object(
            'cannot_delete_group_window')
        self.label_cannot_delete_group = builder.get_object(
            'label_cannot_delete_group')
        # Connect all signals
        builder.connect_signals(self)

if __name__ == "__main__":
    app = GTKUserSetup()
    app.window.show()
    Gtk.main()



#!/usr/bin/python3
# vim:et:sta:sts=4:sw=4:ts=8:tw=79:

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GLib
import os
import subprocess
import threading

# Internationalization
import locale
import gettext
locale.setlocale(locale.LC_ALL, '')
locale.bindtextdomain("gtkiconrefresh", "/usr/share/locale")
gettext.bindtextdomain("gtkiconrefresh", "/usr/share/locale")
gettext.textdomain("gtkiconrefresh")
_ = gettext.gettext


def icon_dirs():
    icondir = '/usr/share/icons/'
    dirs = []
    for i in os.listdir(icondir):
        if os.path.isdir(icondir + i):
            dirs.append(icondir + i)
    return dirs


class GTKIconRefresh:

    def rebuild_icon_cache(self):
        number_of_dirs = len(icon_dirs())
        step = 0.999 / number_of_dirs
        position = 0
        for i in icon_dirs():
            GLib.idle_add(self.update_progress, position)
            cmd = ['gtk-update-icon-cache', '-f', i]
            process = subprocess.Popen(cmd)
            process.wait()
            position = position + step
        self.progressbar.set_fraction(1)
        Gtk.main_quit()

    def update_progress(self, position):
        self.progressbar.set_fraction(position)
        return False

    def gtk_main_quit(self, widget, data=None):
        Gtk.main_quit()

    def on_gtkiconrefresh_delete_event(self, widget, data=None):
        Gtk.main_quit()

    def __init__(self):
        builder = Gtk.Builder()
        builder.set_translation_domain("gtkiconrefresh")
        if os.path.exists('gtkiconrefresh.ui'):
            builder.add_from_file('gtkiconrefresh.ui')
        elif os.path.exists('/usr/share/salixtools/gtkiconrefresh/gtkiconrefresh.ui'):
            builder.add_from_file(
                '/usr/share/salixtools/gtkiconrefresh/gtkiconrefresh.ui')
        self.window = builder.get_object('gtkiconrefresh')
        self.progressbar = builder.get_object('progressbar')

        builder.connect_signals(self)

        thread = threading.Thread(target=self.rebuild_icon_cache)
        thread.daemon = True
        thread.start()

if __name__ == "__main__":
    app = GTKIconRefresh()
    app.window.show()
    Gtk.main()



#!/usr/bin/python3
# vim:et:sta:sts=4:sw=4:ts=8:tw=79:

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
import os
import sys
import re
from ipaddress import ip_address

# Internationalization
import locale
import gettext
locale.setlocale(locale.LC_ALL, '')
locale.bindtextdomain("gtkhostsetup", "/usr/share/locale")
gettext.bindtextdomain("gtkhostsetup", "/usr/share/locale")
gettext.textdomain("gtkhostsetup")
_ = gettext.gettext

hostname_os_file = '/etc/HOSTNAME'
hostname_file = '/etc/hostname_new'
hosts_file = '/etc/hosts'


def get_hostname():
    try:
        if os.access(hostname_file, os.R_OK):
            with open(hostname_file, 'r') as f:
                line = f.readline().replace('\n', '')
        else:
            with open(hostname_os_file, 'r') as f:
                line = f.readline().replace('\n', '')
    except IOError:
        return ['', '']
    if len(line) == 0:
        return ['', '']
    else:
        hostname = line.partition('.')[0]
        domain = line.partition('.')[2]
        return hostname, domain


def get_other_hosts():
    try:
        with open(hosts_file, 'r') as f:
            contents = f.readlines()
    except IOError:
        return [[]]
    hostlines = []
    use_ipv6 = False
    for line in contents:
        temp = line.replace('\t', ' ').replace(
            '\n', '').partition('#')[0].split()
        line = ' '.join(temp)
        if not line == '':
            ip = line.partition(' ')[0]
            hostname = line.partition(' ')[2].partition('.')[0]
            domain = line.partition(' ')[2].partition('.')[2].partition(' ')[0]
            if ip == '::1':
                use_ipv6 = True
            elif ip != '127.0.0.1':
                hostlines.append([ip, hostname, domain])
    return sorted(hostlines), use_ipv6


def check_hostname(ip, hostname, domain):
    error = False
    msg = ''
    if ip == '':
        error = True
        msg = _('The IP address cannot be empty.')
    elif hostname == '':
        error = True
        msg = _('The hostname cannot be empty.')
    elif hostname.startswith('-'):
        error = True
        msg = _('The hostname cannot start with a hyphen.')
    elif hostname.endswith('-'):
        error = True
        msg = _('The hostname cannot end with a hyphen.')
    elif domain.startswith('-'):
        error = True
        msg = _('The domain cannot start with a hyphen.')
    elif domain.endswith('-'):
        error = True
        msg = _('The domain cannot end with a hyphen.')
    if error == False:
        for i in ip:
            try:
                valid_ip = ip_address(ip)
            except ValueError:
                error = True
                msg = _('Not a valid IP address.')
    if error == False:
        for i in hostname:
            if not i in 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-':
                error = True
                msg = _(
                    'The hostname can consist solely of latin letters, numbers and hyphens.')
    if error == False:
        for i in domain:
            if not i in 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-.':
                error = True
                msg = _(
                    'The domain can consist solely of latin letters, numbers, hyphens and dots.')
    return error, msg


def write_hosts(hostname, domain, host_list, use_ipv6):
    with open(hostname_file, 'w') as f:
        if domain == '':
            f.write(hostname + '\n')
        else:
            f.write(hostname + '.' + domain + '\n')
    with open(hosts_file, 'w') as f:
        f.write('#\n')
        f.write('# hosts\tThis file describes a number of hostname-to-address\n')
        f.write('#\t\tmappings for the TCP/IP subsystem. It is mostly\n')
        f.write('#\t\tused at boot time, when no name servers are running.\n')
        f.write('#\t\tOn small systems, this file can be used instead of a\n')
        f.write('#\t\t"named" name server.  Just add the names, addresses\n')
        f.write('#\t\tand any aliases to this file...\n')
        f.write('#\n')
        f.write('# By the way, Arnt Gulbrandsen <agulbra@nvg.unit.no> says that 127.0.0.1\n')
        f.write('# should NEVER be named with the name of the machine. It causes problems\n')
        f.write('# for some (stupid) programs, irc and reputedly talk. :^)\n')
        f.write('#\n')
        f.write('# For loopbacking.\n')
        f.write('127.0.0.1\tlocalhost\n')
        if use_ipv6:
            f.write('::1\t\tlocalhost\n')
        if domain == '':
            f.write('127.0.0.1\t' + hostname + '\n')
            if use_ipv6:
                f.write('::1\t\t' + hostname + '\n')
        else:
            f.write('127.0.0.1\t' + hostname + '.' +
                    domain + ' ' + hostname + '\n')
            if use_ipv6:
                f.write('::1\t\t' + hostname + '.' +
                        domain + ' ' + hostname + '\n')
        f.write('\n')
        for i in host_list:
            ip, hostname, domain = i
            if domain == '':
                f.write(ip + '\t' + hostname + '\n')
            else:
                f.write(ip + '\t' + hostname + '.' +
                        domain + ' ' + hostname + '\n')
        f.write('\n# End of hosts.\n')


class GTKHostSetup:

    def on_button_about_clicked(self, widget, data=None):
        self.aboutdialog.show()

    def on_aboutdialog_response(self, widget, data=None):
        self.aboutdialog.hide()

    def on_aboutdialog_delete_event(self, widget, event):
        self.aboutdialog.hide()
        return True

    def create_host_list(self, widget):
        other_hosts, self.use_ipv6 = get_other_hosts()
        for i in other_hosts:
            self.liststore_hosts.append(i)

    def on_button_ok_clicked(self, widget, data=None):
        this_hostname = self.entry_hostname.get_text()
        this_domain = self.entry_domain.get_text()
        error, message = check_hostname(
            '127.0.0.1', this_hostname, this_domain)
        if error:
            self.label_main_error.set_text(message)
            self.error_main_window.show()
        else:
            host_list = []
            if len(self.liststore_hosts) > 0:
                iter = self.liststore_hosts.get_iter(0)
                while (iter):
                    ip = self.liststore_hosts.get_value(iter, 0)
                    hostname = self.liststore_hosts.get_value(iter, 1)
                    domain = self.liststore_hosts.get_value(iter, 2)
                    host_list.append([ip, hostname, domain])
                    iter = self.liststore_hosts.iter_next(iter)
            write_hosts(self.entry_hostname.get_text(),
                        self.entry_domain.get_text(), sorted(host_list),
                        self.switch_ipv6.get_state())
            Gtk.main_quit()

    def on_button_cancel_clicked(self, widget, data=None):
        Gtk.main_quit()

    def gtk_main_quit(self, widget, data=None):
        Gtk.main_quit()

    def on_button_add_clicked(self, widget, data=None):
        self.add_host_window.show()

    def on_add_host_window_delete_event(self, widget, data=None):
        self.add_host_window.hide()
        return True

    def on_button_edit_clicked(self, widget, data=None):
        try:
            host = self.treeview_hosts.get_selection()
            self.liststore_hosts, iter = host.get_selected()
            self.entry_edit_ip.set_text(
                self.liststore_hosts.get_value(iter, 0))
            self.entry_edit_hostname.set_text(
                self.liststore_hosts.get_value(iter, 1))
            self.entry_edit_domain.set_text(
                self.liststore_hosts.get_value(iter, 2))
            self.edit_host_window.show()
        except TypeError:
            pass

    def on_button_edit_ok_clicked(self, widget, data=None):
        ip = self.entry_edit_ip.get_text()
        hostname = self.entry_edit_hostname.get_text()
        domain = self.entry_edit_domain.get_text()
        error, message = check_hostname(ip, hostname, domain)
        if error:
            self.label_edit_error.set_text(message)
            self.edit_error_window.show()
        else:
            try:
                self.edit_host_window.hide()
                host = self.treeview_hosts.get_selection()
                self.liststore_hosts, iter = host.get_selected()
                self.liststore_hosts.remove(iter)
                self.liststore_hosts.append([ip, hostname, domain])
                self.entry_edit_ip.set_text('')
                self.entry_edit_hostname.set_text('')
                self.entry_edit_domain.set_text('')
                self.liststore_hosts.set_sort_column_id(0, Gtk.SortType.ASCENDING)
            except TypeError:
                pass

    def on_button_edit_cancel_clicked(self, widget, data=None):
        self.edit_host_window.hide()

    def on_edit_host_window_delete_event(self, widget, data=None):
        self.edit_host_window.hide()
        return True

    def on_button_delete_clicked(self, widget, data=None):
        try:
            host = self.treeview_hosts.get_selection()
            self.liststore_hosts, iter = host.get_selected()
            self.liststore_hosts.remove(iter)
        except TypeError:
            pass

    def on_button_add_ok_clicked(self, widget, data=None):
        ip = self.entry_new_ip.get_text()
        hostname = self.entry_new_hostname.get_text()
        domain = self.entry_new_domain.get_text()
        error, message = check_hostname(ip, hostname, domain)
        if error:
            self.label_add_error.set_text(message)
            self.add_error_window.show()
        else:
            self.add_host_window.hide()
            self.liststore_hosts.append([ip, hostname, domain])
            self.entry_new_ip.set_text('')
            self.entry_new_hostname.set_text('')
            self.entry_new_domain.set_text('')
            self.liststore_hosts.set_sort_column_id(0, Gtk.SortType.ASCENDING)

    def on_button_add_cancel_clicked(self, widget, data=None):
        self.add_host_window.hide()

    def on_button_add_error_ok_clicked(self, widget, data=None):
        self.add_error_window.hide()

    def on_add_error_window_delete_event(self, widget, data=None):
        self.add_error_window.hide()
        return True

    def on_button_edit_error_ok_clicked(self, widget, data=None):
        self.edit_error_window.hide()

    def on_edit_error_window_delete_event(self, widget, data=None):
        self.edit_error_window.hide()
        return True

    def on_button_error_main_ok_clicked(self, widget, data=None):
        self.error_main_window.hide()

    def on_error_main_window_delete_event(self, widget, data=None):
        self.error_main_window.hide()
        return True

    def __init__(self):
        builder = Gtk.Builder()
        builder.set_translation_domain("gtkhostsetup")
        if os.path.exists('gtkhostsetup.ui'):
            builder.add_from_file('gtkhostsetup.ui')
        elif os.path.exists('/usr/share/salixtools/gtkhostsetup/gtkhostsetup.ui'):
            builder.add_from_file(
                '/usr/share/salixtools/gtkhostsetup/gtkhostsetup.ui')
        self.window = builder.get_object('gtkhostsetup')

        #
        # Main window
        #
        self.entry_hostname = builder.get_object('entry_hostname')
        self.entry_domain = builder.get_object('entry_domain')
        current_hostname, current_domain = get_hostname()
        self.entry_hostname.set_text(current_hostname)
        self.entry_domain.set_text(current_domain)

        #self.use_ipv6 = False
        self.switch_ipv6 = builder.get_object('switch_ipv6')

        self.treeviewcolumn_ip = builder.get_object('treeviewcolumn_ip')
        self.treeviewcolumn_hostname = builder.get_object(
            'treeviewcolumn_hostname')
        self.treeviewcolumn_domain = builder.get_object(
            'treeviewcolumn_domain')
        self.treeviewcolumn_ip.set_title(_('IP address'))
        self.treeviewcolumn_hostname.set_title(_('Hostname'))
        self.treeviewcolumn_domain.set_title(_('Domain'))

        self.treeview_hosts = builder.get_object('treeview_hosts')
        self.liststore_hosts = builder.get_object('liststore_hosts')
        self.liststore_hosts.clear()
        self.create_host_list(self) # this also sets self.use_ipv6

        self.switch_ipv6.set_state(self.use_ipv6)

        #
        # Add host window
        #
        self.add_host_window = builder.get_object('add_host_window')
        self.entry_new_ip = builder.get_object('entry_new_ip')
        self.entry_new_hostname = builder.get_object('entry_new_hostname')
        self.entry_new_domain = builder.get_object('entry_new_domain')

        #
        # Edit host window
        #
        self.edit_host_window = builder.get_object('edit_host_window')
        self.entry_edit_ip = builder.get_object('entry_edit_ip')
        self.entry_edit_hostname = builder.get_object('entry_edit_hostname')
        self.entry_edit_domain = builder.get_object('entry_edit_domain')

        #
        # Error message in main window
        #
        self.error_main_window = builder.get_object('error_main_window')
        self.label_main_error = builder.get_object('label_main_error')

        #
        # Error message in add window
        #
        self.add_error_window = builder.get_object('add_error_window')
        self.label_add_error = builder.get_object('label_add_error')

        #
        # Error message in edit window
        #
        self.edit_error_window = builder.get_object('edit_error_window')
        self.label_edit_error = builder.get_object('label_edit_error')

        #
        # About dialog
        #
        self.aboutdialog = builder.get_object('aboutdialog')

        builder.connect_signals(self)

if __name__ == "__main__":
    app = GTKHostSetup()
    app.window.show()
    Gtk.main()



#!/usr/bin/python3
# vim:et:sta:sts=4:sw=4:ts=8:tw=79:

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
import os
import sys
import subprocess

# Internationalization
import locale
import gettext
locale.setlocale(locale.LC_ALL, '')
locale.bindtextdomain("gtklocalesetup", "/usr/share/locale")
_ = gettext.gettext

def availablelocales():
    locales = []
    cmd = 'LANG=C locale -cva | grep "^locale\|title |"'
    localeoutput = subprocess.getoutput(cmd).splitlines()
    for i in localeoutput:
        newlocale = False
        if i.startswith('locale:'):
            if 'utf8' in i or 'UTF-8' in i:
                localecode = i.replace('locale:', '').lstrip(
                ).partition('directory:')[0].rstrip().replace('UTF-8', 'utf8')
                utf8locale = True
            else:
                utf8locale = False
        elif 'title |' in i:
            if utf8locale == True:
                localedesc = i.lstrip().replace('title | ', '')
                newlocale = True
        if newlocale == True:
            locales.append([localedesc, localecode])
        locales.sort()
    return locales


def currentlocale():
    localeoutput = subprocess.getoutput("locale").splitlines()
    for i in localeoutput:
        if i.startswith('LANG='):
            locale = i.replace('LANG=', '').replace('UTF-8', 'utf8')
    if 'utf8' not in locale:
        if locale == 'C':
            locale = 'en_US'
        locale = locale + '.utf8'
    return locale


def setlocale(locale):
    cmd = ['localesetup', locale]
    process = subprocess.Popen(cmd)
    process.wait()


class GTKLocaleSetup:

    def on_button_about_clicked(self, widget, data=None):
        self.aboutdialog.show()

    def on_aboutdialog_response(self, widget, data=None):
        self.aboutdialog.hide()

    def on_aboutdialog_delete_event(self, widget, event):
        self.aboutdialog.hide()
        return True

    def on_button_ok_clicked(self, widget, data=None):
        self.window.hide()
        while Gtk.events_pending():
            Gtk.main_iteration()
        selectedlocale = self.localelist.get_selection()
        self.localeliststore, iter = selectedlocale.get_selected()
        setlocale(self.localeliststore.get_value(iter, 1))
        Gtk.main_quit()

    def on_button_cancel_clicked(self, widget, data=None):
        Gtk.main_quit()

    def gtk_main_quit(self, widget, data=None):
        Gtk.main_quit()

    def __init__(self):
        builder = Gtk.Builder()
        builder.set_translation_domain("gtklocalesetup")
        if os.path.exists('gtklocalesetup.ui'):
            builder.add_from_file('gtklocalesetup.ui')
        elif os.path.exists('/usr/share/salixtools/gtklocalesetup/gtklocalesetup.ui'):
            builder.add_from_file(
                '/usr/share/salixtools/gtklocalesetup/gtklocalesetup.ui')
        self.window = builder.get_object('gtklocalesetup')
        self.aboutdialog = builder.get_object('aboutdialog')

        builder.connect_signals(self)
        self.localelist = builder.get_object('localelist')
        self.localeliststore = builder.get_object('localeliststore')
        self.localeliststore.clear()

        cursorpos = 0
        locale = currentlocale()
        for i in availablelocales():
            self.localeliststore.append(i)
            if i[1] == locale:
                self.localelist.set_cursor(cursorpos)
                if cursorpos > 5:
                    self.localelist.scroll_to_cell(cursorpos - 4)
            cursorpos += 1

if __name__ == "__main__":
    app = GTKLocaleSetup()
    app.window.show()
    Gtk.main()



#!/usr/bin/python3
# vim:et:sta:sts=4:sw=4:ts=8:tw=79:


import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk, GLib
import os
import sys
import subprocess
import threading
import time
import platform
import re
import urllib.request, urllib.error, urllib.parse
from textconfig import TextConfig

# Internationalization
import locale
import gettext
locale.setlocale(locale.LC_ALL, '')
locale.bindtextdomain("gtkreposetup", "/usr/share/locale")
gettext.bindtextdomain("gtkreposetup", "/usr/share/locale")
gettext.textdomain("gtkreposetup")
_ = gettext.gettext

canceltask = False

def threaded(f):
    def wrapper(*args):
        t = threading.Thread(target=f, args=args)
        t.daemon = True
        t.start()
    return wrapper

class BreakConnection(Exception):
    '''
    An exception class to help break connections when hitting the cancel
    button.
    '''
    pass

def show_error_dialog(error_msg, parent):
    dialog = Gtk.MessageDialog(parent=parent, flags=0, type=Gtk.MessageType.ERROR,
            buttons=Gtk.ButtonsType.OK, message_format=error_msg)
    dialog.set_title(_('Error'))
    dialog.set_modal(True)
    dialog.set_transient_for(parent)
    dialog.run()
    dialog.destroy()

def get_arch():
    '''
    Determine the CPU architecture. Should be:
    - 'arm' for any arm* system
    - 'i486' for 32bit systems
    - 'x86_64' for 64bit systems
    '''
    m = platform.machine()
    if re.match('i.86$', m):
        return 'i486'
    elif re.match('arm', m):
        return 'arm'
    else:
        return m

def get_salix_version(parent_window):
    '''
    Read the Salix version.
    '''
    p = subprocess.Popen(['lsb_release', '-rs'], stdout=subprocess.PIPE)
    if p.wait() == 0:
        return p.communicate()[0].strip().decode()
    else:
        msg1 = _('lsb_release is not installed.')
        msg2 = _('Exiting.')
        show_error_dialog('%s\n\n%s' % (msg1, msg2), parent_window)
        sys.exit(1)

def get_current_repo(parent_window):
    '''
    Return the currently selected repository. Just runs the command line
    reposetup tool with the -p option to get it.
    '''
    p = subprocess.Popen(['reposetup', '-p'], stdout=subprocess.PIPE)
    if p.wait() == 0:
        return p.communicate()[0].strip().decode()
    # error code 11 is thrown by reposetup when lsb_release is not installed
    elif p.wait() == 11:
        msg1 = _('lsb_release is not installed.')
        msg2 = _('Exiting.')
        show_error_dialog('%s\n\n%s' % (msg1, msg2), parent_window)
        sys.exit(1)
    else:
        return None

def get_repo_list_from_file(parent_window):
    '''
    Reads the list of mirrors from the file they are stored in and returns it
    as a list.
    '''
    repolist = []
    fname = '/usr/share/salixtools/reposetup/repomirrors'
    try:
        with open(fname, 'r') as f:
            while True:
                line = f.readline().rstrip()
                if len(line) == 0:
                    break
                url = line.partition(' ')[0]
                country = line.partition(' ')[2]
                repolist.append([url, country])
    except IOError:
        msg1 = _('Could not read file:')
        msg2 = _('Exiting.')
        show_error_dialog('%s %s\n\n%s' % (msg1, fname, msg2), parent_window)
        sys.exit(1)
    try:
        with open('/etc/salixtools/repos-custom', 'r') as f:
            while True:
                line = f.readline().rstrip()
                if len(line) == 0:
                    break
                if line.startswith('file:///') or line.startswith('http://') \
                        or line.startswith('ftp://'):
                    url = line
                    country = '..'
                    repolist.append([url, country])
    except IOError:
        # not a big deal if the repos-custom file is not there
        pass
    return repolist

def get_slaptget_settings():
    exclude_default = '^aaa_elflibs,^aaa_base,^devs,^glibc.*,^kernel-.*,^udev,' \
                      '^rootuser-settings,^zzz-settings.*'
    fname = '/etc/slapt-get/slapt-getrc'
    xfce418 = False
    try:
        c = TextConfig(fname)
    except IOError:
        msg1 = _('WARNING:')
        msg2 = _('Could not read file:')
        print('%s %s %s' % (msg1, msg2, fname))
        working_dir = '/var/slapt-get'
        exclude = exclude_default
        if arch == 'x86_64':
            exclude = exclude + ',-i?86-'
        else:
            exclude = exclude + ',x86_64'
        custom_repos = []
    else:
        try:
            working_dir = c.get('WORKINGDIR')
        except ValueError:
            working_dir = '/var/slapt-get'
        try:
            exclude = c.get('EXCLUDE')
        except ValueError:
            exclude = exclude_default
            arch = get_arch()
            if arch != 'arm':
                if arch == 'x86_64':
                    exclude = exclude + ',-i?86-'
                else:
                    exclude = exclude + ',x86_64'
        try:
            sources = c.get_all('SOURCE')
            custom_repos = []
            for s in sources:
                if s.endswith(':CUSTOM'):
                    custom_repos.append(s)
                if 'xfce4.18' in s:
                    xfce418 = True
        except ValueError:
            custom_repos = []
    return working_dir, exclude, custom_repos, xfce418

def get_slaptsrc_settings():
    fname = '/etc/slapt-get/slapt-srcrc'
    try:
        c = TextConfig(fname)
    except IOError:
        msg1 = _('WARNING:')
        msg2 = _('Could not read file:')
        print('%s %s %s' % (msg1, msg2, fname))
        build_dir = '/usr/src/slapt-src'
        pkg_ext = 'txz'
    else:
        try:
            build_dir = c.get('BUILDDIR')
        except ValueError:
            build_dir = '/usr/src/slapt-src'
        try:
            pkg_ext = c.get('PKGEXT')
        except ValueError:
            pkg_ext = 'txz'
    return build_dir, pkg_ext

def write_conf(repo, parent_window):
    '''
    Write configuration files.
    '''
    slaptget_working_dir, slaptget_exclude, custom, xfce418 = get_slaptget_settings()
    slaptsrc_build_dir, slaptsrc_pkg_ext = get_slaptsrc_settings()
    arch = get_arch()
    if arch == 'arm':
        slackdir = 'slackwarearm'
    else:
        slackdir = 'slackware'
    version = get_salix_version(parent_window)
    success = True
    #
    # slapt-getrc
    #
    fname = '/etc/slapt-get/slapt-getrc'
    try:
        with open(fname, 'w') as f:
            f.write('# Working directory for local storage/cache.\n')
            f.write('WORKINGDIR={}\n\n'.format(slaptget_working_dir))
            f.write('# Exclude package names and expressions.\n')
            f.write('# To exclude pre and beta packages, add this to the exclude:\n')
            f.write('#   [0-9\_\.\-]{1}pre[0-9\-\.\-]{1}\n')
            f.write('EXCLUDE={}\n\n'.format(slaptget_exclude))
            f.write('# The Slackware repositories, including dependency information\n')
            f.write('SOURCE={r}/{a}/{s}-{v}/:OFFICIAL\n'.format(
                r=repo, a=arch, s=slackdir, v=version))
            f.write('SOURCE={r}/{a}/{s}-{v}/extra/:OFFICIAL\n\n'.format(
                r=repo, a=arch, s=slackdir, v=version))
            f.write('# The Salix repository\n')
            f.write('SOURCE={r}/{a}/{v}/:PREFERRED\n'.format(
                r=repo, a=arch, v=version))
            if xfce418:
                f.write('# The Xfce 4.18 repo for Salix 15.0\n')
                f.write('SOURCE={r}/{a}/xfce4.18-{v}/:PREFERRED\n'.format(
                    r=repo, a=arch, v=version))
            f.write('# And the Salix extra repository\n')
            f.write('SOURCE={r}/{a}/extra-{v}/:OFFICIAL\n\n'.format(
                r=repo, a=arch, v=version))
            f.write('# Local repositories\n')
            f.write('# SOURCE=file:///var/www/packages/:CUSTOM\n')
            for repo in custom:
                f.write('SOURCE={r}\n'.format(r=repo))
    except IOError:
        msg1 = _('Could not write to file:')
        msg2 = _('Exiting.')
        show_error_dialog('%s %s\n\n%s' % (msg1, fname, msg2), parent_window)
        sys.exit(1)
    #
    # slapt-srcrc
    #
    #
    fname = '/etc/slapt-get/slapt-srcrc'
    try:
        with open(fname, 'w') as f:
            f.write('BUILDDIR={}\n'.format(slaptsrc_build_dir))
            f.write('PKGEXT={}\n'.format(slaptsrc_pkg_ext))
            f.write('SOURCE={r}/slkbuild/{v}/\n'.format(
                r=repo, v=version))
            f.write('SOURCE={r}/sbo/{v}/\n'.format(
                r=repo, v=version))
    except IOError:
        msg1 = _('Could not write to file:')
        msg2 = _('Exiting.')
        show_error_dialog('%s %s\n\n%s' % (msg1, fname, msg2), parent_window)
        sys.exit(1)
    return success

def get_mirrors(repo):
    '''
    Retrieves the MIRRORS file from repo.
    '''
    try:
        f = urllib.request.urlopen('{}/MIRRORS'.format(repo))
        mirrors = f.read().splitlines()
        mirrors[:] = [x.decode() for x in mirrors]
    except urllib.error.HTTPError:
        return None
    except urllib.error.URLError:
        return None
    except BreakConnection :
        return None
    except ValueError:
        return None
    except ConnectionResetError:
        return None
    return mirrors

def write_mirrors(mirror_list):
    fname = '/usr/share/salixtools/reposetup/repomirrors'
    try:
        f = open(fname, 'w')
    except IOError:
        msg1 = _('WARNING:')
        msg2 = _('Could not write to file:')
        print("%s %s %s" % (msg1, msg2, fname))
    else:
        for line in mirror_list:
            f.write('{}\n'.format(line))
        f.close()

def update_mirror_list_from_repo(repo):
    '''
    Updates the mirror list from the specified repo.
    '''
    mirrors = get_mirrors(repo)
    if mirrors:
        write_mirrors(mirrors)
        return True
    else:
        return False

class GTKRepoSetup:

    #
    # Main Window
    #
    def on_button_about_clicked(self, widget, data=None):
        self.aboutdialog.show()

    def on_aboutdialog_response(self, widget, data=None):
        self.aboutdialog.hide()

    def on_aboutdialog_delete_event(self, widget, event):
        self.aboutdialog.hide()
        return True

    @threaded
    def on_button_ok_clicked(self, widget, data=None):
        global canceltask
        canceltask = False
        selectedrepo = self.repolist.get_selection()
        self.liststore_repo, iter = selectedrepo.get_selected()
        new_repo = self.liststore_repo.get_value(iter, 0)
        success = True
        update_repos = self.switch_update.get_active()
        error_msg = 'No error'
        if self.current_repo != new_repo:
            GLib.idle_add(self.dialog_update_repos.show)
            # write new configuration files
            ret_conf = write_conf(new_repo, self.window)
            if not ret_conf:
                error_msg = _('Could not write configuration files.')
                success = False
            if success and update_repos:
                # update slapt-getrc
                GLib.idle_add(self.label_update_repos.set_text, _('Updating package information...'))
                self.p = subprocess.Popen(['slapt-get', '--update'])
                while self.p.poll() is None:
                    GLib.idle_add(self.progressbar_update_repos.pulse)
                    time.sleep(0.1)
                retval = self.p.returncode
                if retval != 0:
                    error_msg = _('Could not update package information.')
                    success = False
            if success and update_repos:
                # update slapt-srcrc
                GLib.idle_add(self.label_update_repos.set_text, _('Updating SlackBuild information...'))
                self.p = subprocess.Popen(['slapt-src', '--update'])
                while self.p.poll() is None:
                    GLib.idle_add(self.progressbar_update_repos.pulse)
                    time.sleep(0.1)
                retval = self.p.returncode
                if retval != 0:
                    error_msg = _('Could not update SlackBuild information.')
                    success = False
        GLib.idle_add(self.dialog_update_repos.hide)
        if success and not canceltask:
            Gtk.main_quit()
        elif canceltask:
            GLib.idle_add(self.dialog_update_repos.hide)
        else:
            GLib.idle_add(self.dialog_update_repos.hide)
            GLib.idle_add(show_error_dialog, error_msg, parent=self.window)

    def on_button_update_repos_cancel_clicked(self, widget):
        global canceltask
        canceltask = True
        try:
            self.p.kill()
        except AttributeError: # object has no attribute 'p'
            pass

    def on_button_cancel_clicked(self, widget, data=None):
        Gtk.main_quit()

    def on_gtkreposetup_delete_event(self, widget, event):
        Gtk.main_quit()

    def gtk_main_quit(self, widget, data=None):
        Gtk.main_quit()

    #
    # Update Mirror List Dialog
    #
    @threaded
    def update_mirror_list(self):
        '''
        Tries to update the mirror list from the currently selected repo
        online. If it fails, it reads the local repomirrors file and tries
        to update the mirror list from each one of the repos listed in there.
        '''
        global canceltask
        canceltask = False
        GLib.idle_add(self.dialog_update_list.show)
        repos = [[self.current_repo, None]] + get_repo_list_from_file(self.window)
        for repo in repos:
            GLib.idle_add(self.progressbar_update_list.pulse)
            if not canceltask:
                url = repo[0]
                ret = update_mirror_list_from_repo(url)
                if ret:
                    break
        GLib.idle_add(self.dialog_update_list.hide)
        cursorpos = 0
        selectedpos = 0
        for i in get_repo_list_from_file(self.window):
            self.liststore_repo.append(i)
            if self.current_repo == i[0]:
                selectedpos = cursorpos
            cursorpos += 1
        GLib.idle_add(self.repolist.set_cursor, selectedpos)
        if selectedpos > 5:
            if selectedpos > cursorpos - 5:
                scrollto = cursorpos - 1
            else:
                scrollto = selectedpos + 1
            GLib.idle_add(self.repolist.scroll_to_cell, scrollto)

    def on_button_update_list_cancel_clicked(self, widget):
        GLib.idle_add(self.dialog_update_list.hide)
        global canceltask
        canceltask = True
        raise BreakConnection

    def __init__(self):
        builder = Gtk.Builder()
        builder.set_translation_domain("gtkreposetup")
        if os.path.exists('gtkreposetup.ui'):
            builder.add_from_file('gtkreposetup.ui')
        elif os.path.exists('/usr/share/salixtools/gtkreposetup/gtkreposetup.ui'):
            builder.add_from_file(
                '/usr/share/salixtools/gtkreposetup/gtkreposetup.ui')
        self.window = builder.get_object('gtkreposetup')
        self.switch_update = builder.get_object('switch_update')
        self.switch_update.set_state(True)
        self.aboutdialog = builder.get_object('aboutdialog')

        self.repolist = builder.get_object('repolist')
        self.liststore_repo = builder.get_object('liststore_repo')
        self.liststore_repo.clear()

        # update mirror list dialog
        self.dialog_update_list = builder.get_object('dialog_update_list')
        self.progressbar_update_list = builder.get_object('progressbar_update_list')

        # update package and SlackBuild caches dialog
        self.dialog_update_repos = builder.get_object('dialog_update_repos')
        self.label_update_repos = builder.get_object('label_update_repos')
        self.progressbar_update_repos = builder.get_object('progressbar_update_repos')

        # populate the list of mirrors
        self.current_repo = get_current_repo(parent_window = self.dialog_update_list)

        builder.connect_signals(self)

        self.update_mirror_list()

if __name__ == "__main__":
    app = GTKRepoSetup()
    app.window.show()
    Gtk.main()



#!/usr/bin/python3
# vim:et:sta:sts=4:sw=4:ts=8:tw=79:

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
import os
import sys
import datetime
import subprocess

# Internationalization
import locale
import gettext
locale.setlocale(locale.LC_ALL, '')
locale.bindtextdomain("gtkclocksetup", "/usr/share/locale")
gettext.bindtextdomain("gtkclocksetup", "/usr/share/locale")
gettext.textdomain("gtkclocksetup")
_ = gettext.gettext


def continents():
    continents = ['Africa', 'America', 'Antarctica', 'Asia',
                  'Atlantic', 'Australia', 'Europe', 'Indian',
                  'Pacific', 'US', 'Mexico', 'Chile', 'Mideast',
                  'Canada', 'Brazil', 'Arctic', 'Etc']
    continents.sort()
    return continents


def alltimezones():
    zoneinfo = '/usr/share/zoneinfo/'
    availablecontinents = []
    timezones = []
    for i in continents():
        if os.path.isdir(zoneinfo + i):
            availablecontinents.append(i)
    availablecontinents.sort()

    for i in availablecontinents:
        for file in os.listdir(zoneinfo + i):
            if os.path.isdir(zoneinfo + i + '/' + file):
                for file2 in os.listdir(zoneinfo + i + '/' + file):
                    timezones.append([i, file + '/' + file2])
            else:
                timezones.append([i, file])

    return timezones


def cities(continent):
    cities = []
    for i in alltimezones():
        if i[0] == continent:
            cities.append(i[1])
    cities.sort()
    return cities


def currenttimezone():
    try:
        tz = os.readlink(
            '/etc/localtime').replace('/usr/share/zoneinfo/', '')
    except OSError:
        tz = 'Etc/GMT'
    continent = tz.partition('/')[0]
    location = tz.partition('/')[2]
    return continent, location


def settimezone(continent, location):
    current_continent, current_location= currenttimezone()
    if continent != current_continent or location != current_location:
        cmd = ['rm', '-f', '/etc/localtime']
        process = subprocess.Popen(cmd)
        process.wait()
        cmd = ['ln', '-sf', '/usr/share/zoneinfo/' + continent +
               '/' + location, '/etc/localtime']
        process = subprocess.Popen(cmd)
        process.wait()


def ntpstate():
    if os.access('/etc/rc.d/rc.ntpd', os.X_OK):
        state = True
    else:
        state = False
    return state


def ntppresent():
    if os.path.isfile('/etc/rc.d/rc.ntpd'):
        state = True
    else:
        state = False
    return state


def utcstate():
    utc = False
    try:
        with open('/etc/hardwareclock', 'r') as f:
            while True:
                line = f.readline()
                if len(line) == 0:
                    break
                elif line.rstrip("\n") == 'localtime':
                    utc = False
                    break
                elif line.rstrip("\n") == 'UTC':
                    utc = True
                    break
    except (IOError, FileNotFoundError):
        setutc(False)
    return utc


def setutc(state):
    with open('/etc/hardwareclock', 'w') as f:
        if state == True:
            time = 'UTC'
        else:
            time = 'localtime'
        f.write('# /etc/hardwareclock\n')
        f.write('#\n')
        f.write('# Tells how the hardware clock time is stored.\n')
        f.write('# You should run (gtk)clocksetup or timeconfig to edit this file.\n\n')
        f.write(time + '\n')

def ntp_sync_now():
    cmd = ['/usr/sbin/ntpd', '-gq']
    process = subprocess.Popen(cmd)
    process.wait()
    cmd = ['/sbin/hwclock', '-w']
    process = subprocess.Popen(cmd)
    process.wait()


def setntp(state):
    if state == True:
        cmd = ['/usr/sbin/service', 'start', 'ntpd']
        process = subprocess.Popen(cmd)
        process.wait()
    else:
        cmd = ['/usr/sbin/service', 'stop', 'ntpd']
        process = subprocess.Popen(cmd)
        process.wait()


class GTKClockSetup:

    def on_button_ok_clicked(self, widget, data=None):
        self.window.hide()
        while Gtk.events_pending():
            Gtk.main_iteration()
        if not self.switch_sync.get_active == ntpstate():
            setntp(self.switch_sync.get_active())
            if self.switch_sync.get_active() == False:
                year = "%04d" % (self.calendar.get_date()[0])
                month = "%02d" % (self.calendar.get_date()[1] + 1)
                day = "%02d" % (self.calendar.get_date()[2])
                hour = "%02d" % self.spinbutton_hrs.get_value()
                min = "%02d" % self.spinbutton_min.get_value()
                sec = "%02d" % self.spinbutton_sec.get_value()
                cmd = ['date', '+%Y%m%d', '-s', year + month + day]
                process = subprocess.Popen(cmd)
                process.wait()
                cmd = ['date', '+T', '-s', hour + ':' + min + ':' + sec]
                process = subprocess.Popen(cmd)
                process.wait()
                cmd = ['hwclock', '--systohc']
                process = subprocess.Popen(cmd)
                process.wait()
        if not self.switch_utc.get_active() == utcstate():
            setutc(self.switch_utc.get_active())
        Gtk.main_quit()

    def on_button_cancel_clicked(self, widget, data=None):
        Gtk.main_quit()

    def gtk_main_quit(self, widget, data=None):
        Gtk.main_quit()

    def on_button_timezone_clicked(self, widget, data=None):
        self.tzwindow.show()

    def on_button_sync_now_clicked(self, widget, data=None):
        ntp_sync_now()
        self.update_calendar()
        self.update_time()

    def on_button_about_clicked(self, widget, data=None):
        self.aboutdialog.show()

    def on_aboutdialog_response(self, widget, data=None):
        self.aboutdialog.hide()

    def on_aboutdialog_delete_event(self, widget, event):
        self.aboutdialog.hide()
        return True

    def on_tz_button_ok_clicked(self, widget, data=None):
        pos = self.continentlist.get_cursor()[0][0]
        selectedcontinent = continents()[pos]
        pos = self.locationlist.get_cursor()[0][0]
        selectedlocation = cities(selectedcontinent)[pos]
        self.update_timezone_label(selectedcontinent, selectedlocation)
        settimezone(selectedcontinent, selectedlocation)
        self.tzwindow.hide()

    def on_tz_button_cancel_clicked(self, widget, data=None):
        self.tzwindow.hide()

    def on_timezonewindow_delete_event(self, widget, event):
        self.tzwindow.hide()
        return True

    def on_continentlist_cursor_changed(self, widget, data=None):
        try:
            pos = self.continentlist.get_cursor()[0][0]
        except TypeError:
            pos = 0
        self.locationliststore.clear()
        selectedcontinent = continents()[pos]
        currenttz = currenttimezone()[1]
        count = 0
        set = False
        for i in cities(selectedcontinent):
            self.locationliststore.append([i])
            if i == currenttz:
                set = True
                self.locationlist.set_cursor(count)
                self.locationlist.scroll_to_cell(count)
            count += 1
            if set == False:
                self.locationlist.set_cursor(0)
                self.locationlist.scroll_to_cell(0)

    def update_timezone_label(self, continent, location):
        self.label_timezone.set_label(
            _('Time zone:') + '      ' + continent + '/' + location)

    def on_switch_sync_toggled(self, widget, data=None):
        state = not self.switch_sync.get_active()
        self.label_date.set_sensitive(state)
        self.calendar.set_sensitive(state)
        self.label_time.set_sensitive(state)
        self.spinbutton_hrs.set_sensitive(state)
        self.spinbutton_min.set_sensitive(state)
        self.spinbutton_sec.set_sensitive(state)
        self.button_sync_now.set_sensitive(state)

    def update_calendar(self):
        now = datetime.datetime.now()
        self.calendar.select_month(now.month - 1, now.year)
        self.calendar.select_day(now.day)

    def update_time(self):
        now = datetime.datetime.now()
        self.spinbutton_hrs.set_value(now.hour)
        self.spinbutton_min.set_value(now.minute)
        self.spinbutton_sec.set_value(now.second)

    def __init__(self):
        builder = Gtk.Builder()
        builder.set_translation_domain("gtkclocksetup")
        if os.path.exists('gtkclocksetup.ui'):
            builder.add_from_file('gtkclocksetup.ui')
        elif os.path.exists('/usr/share/salixtools/gtkclocksetup/gtkclocksetup.ui'):
            builder.add_from_file(
                '/usr/share/salixtools/gtkclocksetup/gtkclocksetup.ui')
        self.window = builder.get_object('gtkclocksetup')
        self.tzwindow = builder.get_object('timezonewindow')
        self.tzsyncwindow = builder.get_object('tzsyncwindow')
        self.label_timezone = builder.get_object('label_timezone')
        self.label_timezone.set_label(
            _('Time zone:') + '      ' + currenttimezone()[0] + '/' + currenttimezone()[1])
        self.continentlist = builder.get_object('continentlist')
        self.continentcolumn = builder.get_object('continentcolumn')
        self.continentliststore = builder.get_object('continentliststore')
        self.locationlist = builder.get_object('locationlist')
        self.locationcolumn = builder.get_object('locationcolumn')
        self.locationliststore = builder.get_object('locationliststore')
        self.switch_sync = builder.get_object('switch_sync')
        self.button_sync_now = builder.get_object('button_sync_now')
        self.button_timezone = builder.get_object('button_timezone')
        self.label_date = builder.get_object('label_date')
        self.calendar = builder.get_object('calendar1')
        self.label_time = builder.get_object('label_time')
        self.spinbutton_hrs = builder.get_object('spinbutton_hrs')
        self.spinbutton_min = builder.get_object('spinbutton_min')
        self.spinbutton_sec = builder.get_object('spinbutton_sec')
        self.continentcolumn.set_title(_('General area'))
        self.locationcolumn.set_title(_('Location'))
        self.switch_utc = builder.get_object('switch_utc')
        self.switch_utc.set_active(utcstate())
        self.aboutdialog = builder.get_object('aboutdialog')

        ntp = ntppresent()
        self.switch_sync.set_sensitive(ntp)
        self.button_sync_now.set_sensitive(not ntp)
        ntp = ntpstate()
        self.switch_sync.set_active(ntp)
        self.on_switch_sync_toggled(self)

        currentcontinent = currenttimezone()[0]
        count = 0
        self.continentliststore.clear()
        for i in continents():
            self.continentliststore.append([i])
            if i == currentcontinent:
                self.continentlist.set_cursor(count)
                self.continentlist.scroll_to_cell(count)
            count += 1
        self.on_continentlist_cursor_changed(self)
        builder.connect_signals(self)

        self.update_calendar()
        self.update_time()

if __name__ == "__main__":
    app = GTKClockSetup()
    app.window.show()
    Gtk.main()



#!/usr/bin/python3
# vim:et:sta:sts=4:sw=4:ts=8:tw=79:

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
import os
import sys
import subprocess

# Internationalization
import locale
import gettext
locale.setlocale(locale.LC_ALL, '')
locale.bindtextdomain("gtkkeyboardsetup", "/usr/share/locale")
gettext.bindtextdomain("gtkkeyboardsetup", "/usr/share/locale")
gettext.textdomain("gtkkeyboardsetup")
_ = gettext.gettext


keymapdir = '/usr/share/kbd/keymaps/i386/'


def getkeymap():
    keymap = 'us'
    try:
        with open('/etc/rc.d/rc.keymap', 'r') as f:
            while True:
                line = f.readline().lstrip(' ')
                if len(line) == 0:
                    break
                if line.startswith('/usr/bin/loadkeys'):
                    keymap = line.replace('\n', '').partition(
                        '/usr/bin/loadkeys ')[2].partition('.map')[0].replace('-u ', '')
    except FileNotFoundError:
        # default to 'us' keymap if it's not already set
        keymap = 'us'
    return keymap


def getkeybtype(keymap):
    keybtype = 'qwerty'
    currentkeymap = getkeymap()
    for i in availablekeymaps():
        if i[1] == currentkeymap:
            keybtype = i[0]
            break
    return keybtype


def getnumlock():
    numlock = False
    if os.access('/etc/rc.d/rc.numlock', os.X_OK):
        numlock = True
    return numlock


def ibusavailable():
    ibus = False
    if os.access('/usr/bin/ibus-daemon', os.X_OK):
        if os.path.exists('/etc/profile.d/ibus.sh'):
            ibus = True
    return ibus


def getibus():
    ibus = False
    if ibusavailable() == True:
        if os.access('/etc/profile.d/ibus.sh', os.X_OK):
            ibus = True
    return ibus


def setkeymap(keymap):
    cmd = ['/usr/sbin/keyboardsetup', '-k', keymap]
    process = subprocess.Popen(cmd)
    process.wait()


def setnumlock(numlock):
    if numlock == True:
        state = 'on'
    else:
        state = 'off'
    cmd = ['/usr/sbin/keyboardsetup', '-n', state]
    process = subprocess.Popen(cmd)
    process.wait()


def setibus(ibus):
    if ibus == True:
        state = 'on'
    else:
        state = 'off'
    cmd = ['keyboardsetup', '-i', state]
    process = subprocess.Popen(cmd)
    process.wait()


def availablekeybtypes():
    searchlist = ['azerty', 'dvorak', 'olpc', 'qwerty', 'qwertz']
    keybtypes = []
    for i in searchlist:
        if os.path.isdir(keymapdir + i):
            keybtypes.append(i)
    return keybtypes


def availablekeymaps():
    registered = []
    keymaps = []
    with open('/usr/share/salixtools/keymaps', 'r') as keymapfile:
        while True:
            line = keymapfile.readline()
            if len(line) == 0:
                break
            if not line.startswith('#'):
                registered.append(line.partition('|')[0])
    for keyboard_type in availablekeybtypes():
        for filename in os.listdir(keymapdir + keyboard_type):
            if filename.endswith('.map.gz'):
                key_map = filename.replace('.map.gz', '')
                if key_map in registered:
                    keymaps.append([keyboard_type, key_map])
    keymaps.sort()
    return keymaps


def updatedkeymaplist(keybtype):
    keymaplist = []
    for i in availablekeymaps():
        if i[0] == keybtype:
            keymaplist.append([i[1]])
    return keymaplist


class GTKKeyboardSetup:

    def on_button_about_clicked(self, widget, data=None):
        self.aboutdialog.show()

    def on_aboutdialog_response(self, widget, data=None):
        self.aboutdialog.hide()

    def on_aboutdialog_delete_event(self, widget, event):
        self.aboutdialog.hide()
        return True

    def on_button_ok_clicked(self, widget, data=None):
        self.window.hide()
        while Gtk.events_pending():
            Gtk.main_iteration()
        selectedkeymap = self.keymaplist.get_selection()
        self.keymapliststore, iter = selectedkeymap.get_selected()
        keymap = self.keymapliststore.get_value(iter, 0)
        if not keymap == getkeymap():
            setkeymap(keymap)
        if not self.numlock.get_active() == getnumlock():
            setnumlock(self.numlock.get_active())
        if not self.ibus.get_active() == getibus():
            setibus(self.ibus.get_active())
        Gtk.main_quit()

    def on_button_cancel_clicked(self, widget, data=None):
        Gtk.main_quit()

    def on_keybtypelist_cursor_changed(self, widget, data=None):
        pos = self.keybtypelist.get_cursor()[0][0]
        selectedkeybtype = availablekeybtypes()[pos]
        currentkeymap = getkeymap()
        count = 0
        set = False
        self.keymapliststore.clear()
        for i in updatedkeymaplist(selectedkeybtype):
            self.keymapliststore.append(i)
            if i[0] == currentkeymap:
                set = True
                self.keymaplist.set_cursor(count)
                self.keymaplist.scroll_to_cell(count)
            count += 1
        if set == False:
            self.keymaplist.set_cursor(0)

    def gtk_main_quit(self, widget, data=None):
        Gtk.main_quit()

    def __init__(self):
        builder = Gtk.Builder()
        builder.set_translation_domain("gtkkeyboardsetup")
        if os.path.exists('gtkkeyboardsetup.ui'):
            builder.add_from_file('gtkkeyboardsetup.ui')
        elif os.path.exists('/usr/share/salixtools/gtkkeyboardsetup/gtkkeyboardsetup.ui'):
            builder.add_from_file(
                '/usr/share/salixtools/gtkkeyboardsetup/gtkkeyboardsetup.ui')
        self.window = builder.get_object('gtkkeyboardsetup')

        builder.connect_signals(self)
        self.keybtypelist = builder.get_object('keybtypelist')
        self.keybtypecolumn = builder.get_object('keybtypecolumn')
        self.keybtypecolumn.set_title(_('Keyboard type'))
        self.keybtypeliststore = builder.get_object('keybtypeliststore')
        self.keymaplist = builder.get_object('keymaplist')
        self.keymapcolumn = builder.get_object('keymapcolumn')
        self.keymapcolumn.set_title(_('Keyboard map'))
        self.keymapliststore = builder.get_object('keymapliststore')
        self.numlock = builder.get_object('switch_numlock')
        self.ibus = builder.get_object('switch_ibus')
        self.label_ibus = builder.get_object('label_ibus')
        self.aboutdialog = builder.get_object('aboutdialog')

        currentkeymap = getkeymap()
        currentkeybtype = getkeybtype(currentkeymap)
        count = 0
        self.keybtypeliststore.clear()
        for i in availablekeybtypes():
            self.keybtypeliststore.append([i])
            if i == currentkeybtype:
                self.keybtypelist.set_cursor(count)
            count += 1

        if ibusavailable() == True:
            self.ibus.set_active(getibus())
        else:
            self.label_ibus.set_sensitive(False)
            self.ibus.set_sensitive(False)
            self.ibus.set_active(False)

        self.numlock.set_active(getnumlock())

if __name__ == "__main__":
    app = GTKKeyboardSetup()
    app.window.show()
    Gtk.main()



#!/usr/bin/python3
# vim:et:sta:sts=4:sw=4:ts=8:tw=79:

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
import subprocess
import os
import sys

# Internationalization
import locale
import gettext
locale.setlocale(locale.LC_ALL, '')
locale.bindtextdomain("gtkservicesetup", "/usr/share/locale")
gettext.bindtextdomain("gtkservicesetup", "/usr/share/locale")
gettext.textdomain("gtkservicesetup")
_ = gettext.gettext

def blacklisted():
    fname = '/usr/share/salixtools/servicesetup/service-blacklist'
    with open(fname, 'r') as f:
        servicestrings = []
        while True:
            line = f.readline().replace('\n', '')
            if len(line) == 0:
                break
            servicestrings.append(line)
    return servicestrings


def isblacklisted(service):
    val = False
    for i in blacklisted():
        if i in service:
            val = True
    return val


def isenabled(service):
    if os.access('/etc/rc.d/' + service, os.X_OK):
        enabled = True
    else:
        enabled = False
    return enabled


def servicedesc(service):
    description = _('The {0} service').format(service[3:])
    filelist = os.listdir('/etc/rc.d/desc.d')
    for i in filelist:
        with open('/etc/rc.d/desc.d/' + i, 'r') as f:
            while True:
                line = f.readline()
                if len(line) == 0:
                    break
                elif 'rc.' + line.partition(':')[0] == service:
                    description = line.partition(':')[2].partition(':')[0]
                    break
    return description


def availableservices():
    filelist = os.listdir('/etc/rc.d')
    services = []
    for i in filelist:
        if os.path.isfile('/etc/rc.d/' + i):
            if not isblacklisted(i):
                services.append(i)
    services.sort()
    return services


def serviceslist():
    list = []
    for i in availableservices():
        list.append([isenabled(i), i.replace(
            'rc.', ''), servicedesc(i), False])
    return list


def enableservice(service):
    cmd = ['/usr/sbin/service', 'start', service]
    process = subprocess.Popen(cmd)
    process.wait()


def disableservice(service):
    cmd = ['/usr/sbin/service', 'stop', service]
    process = subprocess.Popen(cmd)
    process.wait()


class GTKServiceSetup:

    def on_button_about_clicked(self, widget, data=None):
        self.aboutdialog.show()

    def on_aboutdialog_response(self, widget, data=None):
        self.aboutdialog.hide()

    def on_aboutdialog_delete_event(self, widget, event):
        self.aboutdialog.hide()
        return True

    def on_button_ok_clicked(self, widget, data=None):
        self.window.hide()
        while Gtk.events_pending():
            Gtk.main_iteration()
        for i in self.servicesliststore:
            if i[3] == True:
                if i[0] == True:
                    enableservice(i[1])
                else:
                    disableservice(i[1])
        Gtk.main_quit()

    def on_button_cancel_clicked(self, widget, data=None):
        Gtk.main_quit()

    def on_checkbox_toggled(self, widget, data=None):
        selectedservice = self.serviceslist.get_selection()
        self.servicesliststore, iter = selectedservice.get_selected()
        selectedservicename = self.servicesliststore.get_value(iter, 1)
        for i in self.servicesliststore:
            if i[1] == selectedservicename:
                i[3] = True
                if i[0] == 0:
                    i[0] = 1
                else:
                    i[0] = 0

    def gtk_main_quit(self, widget, data=None):
        Gtk.main_quit()

    def __init__(self):
        builder = Gtk.Builder()
        builder.set_translation_domain("gtkservicesetup")
        if os.path.exists('gtkservicesetup.ui'):
            builder.add_from_file('gtkservicesetup.ui')
        elif os.path.exists('/usr/share/salixtools/gtkservicesetup/gtkservicesetup.ui'):
            builder.add_from_file(
                '/usr/share/salixtools/gtkservicesetup/gtkservicesetup.ui')
        self.window = builder.get_object('gtkservicesetup')
        self.aboutdialog = builder.get_object('aboutdialog')

        builder.connect_signals(self)
        self.serviceslist = builder.get_object('serviceslist')
        self.servicesliststore = builder.get_object('servicesliststore')
        self.columnenabled = builder.get_object('columnEnabled')
        self.columnenabled.set_title(_('Enabled'))
        self.columnservicename = builder.get_object('columnServiceName')
        self.columnservicename.set_title(_('Service name'))
        self.columnservicedesc = builder.get_object('columnServiceDesc')
        self.columnservicedesc.set_title(_('Service description'))
        self.servicesliststore.clear()
        for i in serviceslist():
            self.servicesliststore.append(i)

if __name__ == "__main__":
    app = GTKServiceSetup()
    app.window.show()
    Gtk.main()
